#Imports
from PIL import Image, ImageFilter
from PIL.Image import fromarray
import numpy as np

class RescaleImage(object):
    '''
        Rescales the image to a given size.
    '''

    def __init__(self, output_size):
        '''
            Arguments:
            output_size (tuple or int): Desired output size. If tuple, output is
            matched to output_size. If int, smaller of image edges is matched
            to output_size keeping aspect ratio the same.
        '''

        # Write your code here
        self.output_size=output_size

    def __call__(self, image):
        '''
            Arguments:
            image (numpy array or PIL image)

            Returns:
            image (numpy array or PIL image)

            Note: You do not need to resize the bounding boxes. ONLY RESIZE THE IMAGE.
        '''

        # Write your code here
        initH=image.size[0]
        initW=image.size[1]
        if isinstance(self.output_size, int):
            if initH>=initW:
                finW = self.output_size*initW
                finH = round((initH*finW)/initW)
            else:
                finH = self.output_size
                finW = round((initW*finH)/initH)
            img=image.resize((int(finW), int(finH)))
        elif isinstance(self.output_size, float):
            finW = initW*self.output_size
            finH = initH*self.output_size
            img=image.resize((int(finH), int(finW)))
        else:
            img = image.resize(self.output_size)
        return img



#Imports
from PIL import ImageDraw, Image, ImageFont
def plot_boxes(image, path, boxes, filename): # Write the required arguments

  # The function should plot the predicted boxes on the images and save them.
  # Tip: keep the dimensions of the output image less than 800 to avoid RAM crashes.
  font = ImageFont.load_default()
  img = Image.fromarray(image)
  drawImg = ImageDraw.Draw(img)
  for i,box in enumerate(boxes[:5]):
    tupleBox=tuple(box)
    drawImg.rectangle(tupleBox, outline="blue")
    drawImg.text((box[0][0], box[0][1]-25), filename[i], fill = "red", font = font, stroke_width = 25)
  pth = path
  img.save(fp=pth)
  return img  
  



#Imports
from PIL import Image, ImageFilter
from PIL.Image import fromarray
import numpy as np


class RotateImage(object):
    '''
        Rotates the image about the centre of the image.
    '''

    def __init__(self, degrees):
        '''
            Arguments:
            degrees: rotation degree.
            
        '''
        
        # Write your code here
        self.degrees=degrees

    def __call__(self, sample):
        '''
            Arguments:
            image (numpy array or PIL image)

            Returns:
            image (numpy array or PIL image)
        '''

        # Write your code here
        img = sample.rotate(self.degrees, Image.NEAREST, expand=1)
        return img


#Imports
import numpy as np
from PIL import Image
from PIL.Image import fromarray
import random

class CropImage(object):
    '''
        Performs either random cropping or center cropping.
    '''

    def __init__(self, shape, crop_type='center'):
        '''
            Arguments:
            shape: output shape of the crop (h, w)
            crop_type: center crop or random crop. Default: center
        '''

        # Write your code here
        self.shape=shape
        self.crop_type=crop_type

    def __call__(self, image):
        '''
            Arguments:
            image (numpy array or PIL image)

            Returns:
            image (numpy array or PIL image)
        '''

        # Write your code here
        imgH=image.size[0]
        imgW=image.size[1]
        cropH=self.shape[0]
        cropW=self.shape[1]
        if str(self.crop_type) == "center":
            top = round((imgH-cropH)/2)
            left = round((imgW-cropW)/2)
            right = cropW + left
            down = cropH + top
        else:
            top=random.randint(0, imgH-cropH)
            left=random.randint(0, imgW-cropW)
            right = cropW + left
            down = cropH + top
        img = image.crop((left, top, right, down))
        return img

        

 


#Imports
import jsonlines
from PIL import Image
import numpy as np

class Dataset(object):
    '''
        A class for the dataset that will return data items as per the given index
    '''

    def __init__(self, annotation_file, transforms = None):
        '''
            Arguments:
            annotation_file: path to the annotation file
            transforms: list of transforms (class instances)
                        For instance, [<class 'RandomCrop'>, <class 'Rotate'>]
        '''
        self.annotation_file=annotation_file
        if transforms==None:
            self.transforms = []
        else:
            self.transforms = transforms
        self.image_paths = []
        self.bboxes = []
        with jsonlines.open(annotation_file) as fl:
            for line in fl.iter():
                self.image_paths.append(line['img_fn'])
                self.bboxes.append(line['bboxes'])
        

    def __len__(self):
        '''
            return the number of data points in the dataset
        '''
        return len(self.annotation_file) 

        

    def __getitem__(self, idx):
        '''
            return the dataset element for the index: "idx"
            Arguments:
                idx: index of the data element.

            Returns: A dictionary with:
                image: image (in the form of a numpy array) (shape: (3, H, W))
                gt_bboxes: N X 5 array where N is the number of bounding boxes, each 
                            consisting of [class, x1, y1, x2, y2]
                            x1 and x2 lie between 0 and width of the image,
                            y1 and y2 lie between 0 and height of the image.

            You need to do the following, 
            1. Extract the correct annotation using the idx provided.
            2. Read the image and convert it into a numpy array (wont be necessary
                with some libraries). The shape of the array would be (3, H, W).
            3. Scale the values in the array to be with [0, 1].
            4. Create a dictonary with both the image and annotations
            4. Perform the desired transformations.
            5. Return the transformed image and annotations as specified.
        '''
        edit_path = "./data/"
        image = Image.open(edit_path + self.image_paths[idx])
        
        for x in self.transforms:
            image = x(image)
            
        gt_bboxes = []
        for x in self.bboxes[idx]:
            arr = []
            arr.append(x['category'])
            arr = [*arr, *x['bbox']]
            gt_bboxes.append(arr)
        
        imageArr = np.array(image)
        imageArr = imageArr.transpose(2,0,1).astype('float32')
        imageArr/=255.0
        return {"image" : imageArr, "gt_bboxes" : gt_bboxes}
        
        

        



from .model import ObjectDetectionModel
from img_detector_boxes_red.data.transforms import BlurImage
from img_detector_boxes_red.data.transforms import CropImage
from img_detector_boxes_red.data.transforms import FlipImage
from img_detector_boxes_red.data.transforms import RescaleImage
from img_detector_boxes_red.data.transforms import RotateImage
from img_detector_boxes_red.data import Dataset
from img_detector_boxes_red.analysis import plot_boxes



#Imports
from PIL import Image, ImageFilter
from PIL.Image import fromarray
import numpy as np


class BlurImage(object):
    '''
        Applies Gaussian Blur on the image.
    '''

    def __init__(self, radius):
        '''
            Arguments:
            radius (int): radius to blur
        '''
        

        # Write your code here
        self.radius=int(radius)
        

    def __call__(self, image):
        '''
            Arguments:
            image (numpy array or PIL Image)

            Returns:
            image (numpy array or PIL Image)
        '''

        # Write your code here
        img = image.filter(ImageFilter.GaussianBlur(radius=self.radius))
        return img



#Imports
from PIL import Image, ImageFilter
from PIL.Image import fromarray
import numpy as np

class FlipImage(object):
    '''
        Flips the image.
    '''

    def __init__(self, flip_type='horizontal'):
        '''
            Arguments:
            flip_type: 'horizontal' or 'vertical' Default: 'horizontal'
        '''

        # Write your code here
        self.flip_type=str(flip_type)

        
    def __call__(self, image):
        '''
            Arguments:
            image (numpy array or PIL image)

            Returns:
            image (numpy array or PIL image)
        '''

        # Write your code here
        if self.flip_type=="horizontal":
            img = image.transpose(Image.FLIP_TOP_BOTTOM)
        else:
            img = image.transpose(Image.FLIP_LEFT_RIGHT)
        return img